package ch.psi.daq.queryrest.response.msgpack;

import java.io.OutputStream;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.ServletResponse;

import org.apache.commons.lang3.tuple.Triple;
import org.msgpack.jackson.dataformat.MessagePackFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.json.ChannelName;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.backend.BackendQuery;
import ch.psi.daq.domain.query.operation.Aggregation;
import ch.psi.daq.domain.query.response.Response;
import ch.psi.daq.queryrest.config.QueryRestConfig;
import ch.psi.daq.queryrest.response.ResponseStreamWriter;
import ch.psi.daq.queryrest.response.json.JSONTableResponseStreamWriter;

/**
 * Takes a Java 8 stream and writes it to the output stream provided by the {@link ServletResponse}
 * of the current request.
 */
public class MsgPackTableResponseStreamWriter implements ResponseStreamWriter, ApplicationContextAware {
   private ObjectMapper mapper;
   private MessagePackFactory factory;
   // In case ArchiverAppliance had several events within the 10ms mapping interval, return these
   // aggregations
   private Set<String> defaultResponseAggregationsStr;

   @SuppressWarnings("unchecked")
   @Override
   public void setApplicationContext(ApplicationContext context) throws BeansException {
      final Backend backend = context.getBean(DomainConfig.BEAN_NAME_BACKEND_DEFAULT, Backend.class);
      context = backend.getApplicationContext();

      mapper = context.getBean(DomainConfig.BEAN_NAME_OBJECT_MAPPER, ObjectMapper.class);
      factory = context.getBean(QueryRestConfig.BEAN_NAME_MSG_PACK_FACTORY, MessagePackFactory.class);

      final Set<Aggregation> defaultResponseAggregations =
            context.getBean(QueryRestConfig.BEAN_NAME_DEFAULT_RESPONSE_AGGREGATIONS, Set.class);;
      defaultResponseAggregationsStr =
            defaultResponseAggregations.stream().map(Aggregation::name)
                  .collect(Collectors.toCollection(LinkedHashSet::new));
   }

   @Override
   public void respond(final List<Entry<DAQQueryElement, Stream<Triple<BackendQuery, ChannelName, ?>>>> results,
         final OutputStream out, final Response response) throws Exception {
      JSONTableResponseStreamWriter.respond(factory, mapper, defaultResponseAggregationsStr, results, out, response);
   }
}
